/* valastruct.vala
 *
 * Copyright (C) 2006  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

#include "valastruct.h"
#include "valamethod.h"
#include "valasourcereference.h"
#include "valatypeparameter.h"
#include "valaconstant.h"
#include "valafield.h"
#include "valacodevisitor.h"
#include "valanamespace.h"
#include "valaattribute.h"
#include "valanamedargument.h"
#include "valaliteralexpression.h"
#include "valaliteral.h"
#include "valastringliteral.h"
#include "valareport.h"
#include "valasymbol.h"

struct _ValaStructPrivate {
	GList* type_parameters;
	GList* constants;
	GList* fields;
	GList* methods;
	char* cname;
	char* dup_function;
	char* free_function;
	char* type_id;
	char* lower_case_csuffix;
	gboolean reference_type;
	char* marshaller_type_name;
	ValaMethod* _default_construction_method;
};
#define VALA_STRUCT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_STRUCT, ValaStructPrivate))
enum  {
	VALA_STRUCT_DUMMY_PROPERTY,
	VALA_STRUCT_DEFAULT_CONSTRUCTION_METHOD,
};
static void vala_struct_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor);
static char* vala_struct_real_get_cname (ValaDataType* base);
static void vala_struct_set_cname (ValaStruct* self, char* cname);
static char* vala_struct_get_lower_case_csuffix (ValaStruct* self);
static void vala_struct_set_lower_case_csuffix (ValaStruct* self, char* csuffix);
static char* vala_struct_real_get_lower_case_cname (ValaDataType* base, char* infix);
static char* vala_struct_real_get_upper_case_cname (ValaDataType* base, char* infix);
static gboolean vala_struct_real_is_reference_type (ValaDataType* base);
static void vala_struct_process_ccode_attribute (ValaStruct* self, ValaAttribute* a);
static void vala_struct_process_ref_type_attribute (ValaStruct* self, ValaAttribute* a);
static gboolean vala_struct_real_is_reference_counting (ValaDataType* base);
static char* vala_struct_real_get_dup_function (ValaDataType* base);
static char* vala_struct_real_get_free_function (ValaDataType* base);
static void vala_struct_set_free_function (ValaStruct* self, char* name);
static char* vala_struct_real_get_type_id (ValaDataType* base);
static void vala_struct_set_type_id (ValaStruct* self, char* name);
static char* vala_struct_real_get_marshaller_type_name (ValaDataType* base);
static void vala_struct_set_marshaller_type_name (ValaStruct* self, char* name);
static void vala_struct_dispose (GObject * obj);

/**
 * Creates a new struct.
 *
 * @param name   type name
 * @param source reference to source code
 * @return       newly created struct
 */
ValaStruct* vala_struct_new (char* _name, ValaSourceReference* source)
{
	g_return_val_if_fail (_name != NULL, NULL);
	g_return_val_if_fail (source == NULL || VALA_IS_SOURCE_REFERENCE (source), NULL);
	GParameter * __params = g_new0 (GParameter, 2);
	GParameter * __params_it = __params;
	(__params_it->name = "name", g_value_init (&__params_it->value, G_TYPE_STRING), g_value_set_string (&__params_it->value, _name), __params_it++);
	(__params_it->name = "source-reference", g_value_init (&__params_it->value, VALA_TYPE_SOURCE_REFERENCE), g_value_set_object (&__params_it->value, source), __params_it++);
	ValaStruct * self = g_object_newv (VALA_TYPE_STRUCT, __params_it - __params, __params);
	return self;
}


/**
 * Appends the specified parameter to the list of type parameters.
 *
 * @param p a type parameter
 */
void vala_struct_add_type_parameter (ValaStruct* self, ValaTypeParameter* p)
{
	g_return_if_fail (VALA_IS_STRUCT (self));
	g_return_if_fail (VALA_IS_TYPE_PARAMETER (p));
	self->priv->type_parameters = g_list_append (self->priv->type_parameters, g_object_ref (p));
	p->type = VALA_DATA_TYPE (self);
}


/**
 * Adds the specified constant as a member to this struct.
 *
 * @param c a constant
 */
void vala_struct_add_constant (ValaStruct* self, ValaConstant* c)
{
	g_return_if_fail (VALA_IS_STRUCT (self));
	g_return_if_fail (VALA_IS_CONSTANT (c));
	self->priv->constants = g_list_append (self->priv->constants, g_object_ref (c));
}


/**
 * Adds the specified field as a member to this struct.
 *
 * @param f a field
 */
void vala_struct_add_field (ValaStruct* self, ValaField* f)
{
	g_return_if_fail (VALA_IS_STRUCT (self));
	g_return_if_fail (VALA_IS_FIELD (f));
	self->priv->fields = g_list_append (self->priv->fields, g_object_ref (f));
}


/**
 * Returns a copy of the list of fields.
 *
 * @return list of fields
 */
GList* vala_struct_get_fields (ValaStruct* self)
{
	g_return_val_if_fail (VALA_IS_STRUCT (self), NULL);
	return g_list_copy (self->priv->fields);
}


/**
 * Adds the specified method as a member to this struct.
 *
 * @param m a method
 */
void vala_struct_add_method (ValaStruct* self, ValaMethod* m)
{
	g_return_if_fail (VALA_IS_STRUCT (self));
	g_return_if_fail (VALA_IS_METHOD (m));
	g_return_if_fail (m != NULL);
	self->priv->methods = g_list_append (self->priv->methods, g_object_ref (m));
}


/**
 * Returns a copy of the list of methods.
 *
 * @return list of methods
 */
GList* vala_struct_get_methods (ValaStruct* self)
{
	g_return_val_if_fail (VALA_IS_STRUCT (self), NULL);
	return g_list_copy (self->priv->methods);
}


static void vala_struct_real_accept (ValaCodeNode* base, ValaCodeVisitor* visitor)
{
	ValaStruct * self = VALA_STRUCT (base);
	g_return_if_fail (VALA_IS_CODE_VISITOR (visitor));
	vala_code_visitor_visit_begin_struct (visitor, self);
	{
		GList* p_it;
		for (p_it = self->priv->type_parameters; p_it != NULL; p_it = p_it->next) {
			ValaTypeParameter* p = p_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (p), visitor);
			}
		}
	}
	{
		GList* f_it;
		for (f_it = self->priv->fields; f_it != NULL; f_it = f_it->next) {
			ValaField* f = f_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (f), visitor);
			}
		}
	}
	{
		GList* c_it;
		for (c_it = self->priv->constants; c_it != NULL; c_it = c_it->next) {
			ValaConstant* c = c_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (c), visitor);
			}
		}
	}
	{
		GList* m_it;
		for (m_it = self->priv->methods; m_it != NULL; m_it = m_it->next) {
			ValaMethod* m = m_it->data;
			{
				vala_code_node_accept (VALA_CODE_NODE (m), visitor);
			}
		}
	}
	vala_code_visitor_visit_end_struct (visitor, self);
}


static char* vala_struct_real_get_cname (ValaDataType* base)
{
	ValaStruct * self = VALA_STRUCT (base);
	if (self->priv->cname == NULL) {
		char* __temp2 = NULL;
		self->priv->cname = (__temp2 = g_strdup_printf ("%s%s", vala_namespace_get_cprefix (VALA_DATA_TYPE (self)->namespace), vala_data_type_get_name (VALA_DATA_TYPE (self))), (self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL))), __temp2);
	}
	return self->priv->cname;
}


static void vala_struct_set_cname (ValaStruct* self, char* cname)
{
	g_return_if_fail (VALA_IS_STRUCT (self));
	g_return_if_fail (cname != NULL);
	char* __temp4 = NULL;
	self->priv->cname = (__temp4 = g_strdup (cname), (self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL))), __temp4);
}


static char* vala_struct_get_lower_case_csuffix (ValaStruct* self)
{
	g_return_val_if_fail (VALA_IS_STRUCT (self), NULL);
	if (self->priv->lower_case_csuffix == NULL) {
		char* __temp5 = NULL;
		self->priv->lower_case_csuffix = (__temp5 = vala_namespace_camel_case_to_lower_case (vala_data_type_get_name (VALA_DATA_TYPE (self))), (self->priv->lower_case_csuffix == NULL ? NULL : (self->priv->lower_case_csuffix = (g_free (self->priv->lower_case_csuffix), NULL))), __temp5);
	}
	return self->priv->lower_case_csuffix;
}


static void vala_struct_set_lower_case_csuffix (ValaStruct* self, char* csuffix)
{
	g_return_if_fail (VALA_IS_STRUCT (self));
	g_return_if_fail (csuffix != NULL);
	char* __temp7 = NULL;
	self->priv->lower_case_csuffix = (__temp7 = g_strdup (csuffix), (self->priv->lower_case_csuffix == NULL ? NULL : (self->priv->lower_case_csuffix = (g_free (self->priv->lower_case_csuffix), NULL))), __temp7);
}


static char* vala_struct_real_get_lower_case_cname (ValaDataType* base, char* infix)
{
	ValaStruct * self = VALA_STRUCT (base);
	if (infix == NULL) {
		infix = "";
	}
	return g_strdup_printf ("%s%s%s", vala_namespace_get_lower_case_cprefix (VALA_DATA_TYPE (self)->namespace), infix, vala_struct_get_lower_case_csuffix (self));
}


static char* vala_struct_real_get_upper_case_cname (ValaDataType* base, char* infix)
{
	ValaStruct * self = VALA_STRUCT (base);
	char* __temp9 = NULL;
	char* __temp10 = NULL;
	return (__temp10 = g_utf8_strup ((__temp9 = vala_data_type_get_lower_case_cname (VALA_DATA_TYPE (self), infix)), -1), (__temp9 == NULL ? NULL : (__temp9 = (g_free (__temp9), NULL))), __temp10);
}


static gboolean vala_struct_real_is_reference_type (ValaDataType* base)
{
	ValaStruct * self = VALA_STRUCT (base);
	return self->priv->reference_type;
}


static void vala_struct_process_ccode_attribute (ValaStruct* self, ValaAttribute* a)
{
	g_return_if_fail (VALA_IS_STRUCT (self));
	g_return_if_fail (VALA_IS_ATTRIBUTE (a));
	{
		GList* arg_it;
		for (arg_it = a->args; arg_it != NULL; arg_it = arg_it->next) {
			ValaNamedArgument* arg = arg_it->data;
			{
				if (g_utf8_collate (vala_named_argument_get_name (arg), "cname") == 0) {
					/* this will already be checked during semantic analysis */
					if (VALA_IS_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))) {
						ValaLiteral* lit = vala_literal_expression_get_literal ((VALA_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))));
						if (VALA_IS_STRING_LITERAL (lit)) {
							char* __temp13 = NULL;
							vala_struct_set_cname (self, (__temp13 = vala_string_literal_eval ((VALA_STRING_LITERAL (lit)))));
							(__temp13 == NULL ? NULL : (__temp13 = (g_free (__temp13), NULL)));
						}
					}
				} else {
					if (g_utf8_collate (vala_named_argument_get_name (arg), "cheader_filename") == 0) {
						/* this will already be checked during semantic analysis */
						if (VALA_IS_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))) {
							ValaLiteral* lit = vala_literal_expression_get_literal ((VALA_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))));
							if (VALA_IS_STRING_LITERAL (lit)) {
								char* val = vala_string_literal_eval ((VALA_STRING_LITERAL (lit)));
								{
									char** filename_it;
									for (filename_it = g_strsplit (val, ",", 0); *filename_it != NULL; filename_it = filename_it + 1) {
										char* filename = *filename_it;
										{
											vala_data_type_add_cheader_filename (VALA_DATA_TYPE (self), filename);
										}
									}
								}
								(val == NULL ? NULL : (val = (g_free (val), NULL)));
							}
						}
					} else {
						if (g_utf8_collate (vala_named_argument_get_name (arg), "type_id") == 0) {
							/* this will already be checked during semantic analysis */
							if (VALA_IS_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))) {
								ValaLiteral* lit = vala_literal_expression_get_literal ((VALA_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))));
								if (VALA_IS_STRING_LITERAL (lit)) {
									char* __temp14 = NULL;
									vala_struct_set_type_id (self, (__temp14 = vala_string_literal_eval ((VALA_STRING_LITERAL (lit)))));
									(__temp14 == NULL ? NULL : (__temp14 = (g_free (__temp14), NULL)));
								}
							}
						} else {
							if (g_utf8_collate (vala_named_argument_get_name (arg), "marshaller_type_name") == 0) {
								/* this will already be checked during semantic analysis */
								if (VALA_IS_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))) {
									ValaLiteral* lit = vala_literal_expression_get_literal ((VALA_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))));
									if (VALA_IS_STRING_LITERAL (lit)) {
										char* __temp15 = NULL;
										vala_struct_set_marshaller_type_name (self, (__temp15 = vala_string_literal_eval ((VALA_STRING_LITERAL (lit)))));
										(__temp15 == NULL ? NULL : (__temp15 = (g_free (__temp15), NULL)));
									}
								}
							}
						}
					}
				}
			}
		}
	}
}


static void vala_struct_process_ref_type_attribute (ValaStruct* self, ValaAttribute* a)
{
	g_return_if_fail (VALA_IS_STRUCT (self));
	g_return_if_fail (VALA_IS_ATTRIBUTE (a));
	self->priv->reference_type = TRUE;
	{
		GList* arg_it;
		for (arg_it = a->args; arg_it != NULL; arg_it = arg_it->next) {
			ValaNamedArgument* arg = arg_it->data;
			{
				if (g_utf8_collate (vala_named_argument_get_name (arg), "dup_function") == 0) {
					/* this will already be checked during semantic analysis */
					if (VALA_IS_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))) {
						ValaLiteral* lit = vala_literal_expression_get_literal ((VALA_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))));
						if (VALA_IS_STRING_LITERAL (lit)) {
							char* __temp16 = NULL;
							vala_struct_set_dup_function (self, (__temp16 = vala_string_literal_eval ((VALA_STRING_LITERAL (lit)))));
							(__temp16 == NULL ? NULL : (__temp16 = (g_free (__temp16), NULL)));
						}
					}
				} else {
					if (g_utf8_collate (vala_named_argument_get_name (arg), "free_function") == 0) {
						/* this will already be checked during semantic analysis */
						if (VALA_IS_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))) {
							ValaLiteral* lit = vala_literal_expression_get_literal ((VALA_LITERAL_EXPRESSION (vala_named_argument_get_argument (arg))));
							if (VALA_IS_STRING_LITERAL (lit)) {
								char* __temp17 = NULL;
								vala_struct_set_free_function (self, (__temp17 = vala_string_literal_eval ((VALA_STRING_LITERAL (lit)))));
								(__temp17 == NULL ? NULL : (__temp17 = (g_free (__temp17), NULL)));
							}
						}
					}
				}
			}
		}
	}
}


/**
 * Process all associated attributes.
 */
void vala_struct_process_attributes (ValaStruct* self)
{
	g_return_if_fail (VALA_IS_STRUCT (self));
	{
		GList* a_it;
		for (a_it = VALA_CODE_NODE (self)->attributes; a_it != NULL; a_it = a_it->next) {
			ValaAttribute* a = a_it->data;
			{
				if (g_utf8_collate (vala_attribute_get_name (a), "CCode") == 0) {
					vala_struct_process_ccode_attribute (self, a);
				} else {
					if (g_utf8_collate (vala_attribute_get_name (a), "ReferenceType") == 0) {
						vala_struct_process_ref_type_attribute (self, a);
					}
				}
			}
		}
	}
}


static gboolean vala_struct_real_is_reference_counting (ValaDataType* base)
{
	ValaStruct * self = VALA_STRUCT (base);
	return FALSE;
}


static char* vala_struct_real_get_dup_function (ValaDataType* base)
{
	ValaStruct * self = VALA_STRUCT (base);
	if (self->priv->dup_function == NULL) {
		char* __temp20 = NULL;
		char* __temp19 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (self)), (__temp20 = g_strdup_printf ("The type `%s` doesn't contain a copy function", (__temp19 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (self)))))));
		(__temp20 == NULL ? NULL : (__temp20 = (g_free (__temp20), NULL)));
		(__temp19 == NULL ? NULL : (__temp19 = (g_free (__temp19), NULL)));
	}
	return self->priv->dup_function;
}


void vala_struct_set_dup_function (ValaStruct* self, char* name)
{
	g_return_if_fail (VALA_IS_STRUCT (self));
	g_return_if_fail (name != NULL);
	char* __temp22 = NULL;
	self->priv->dup_function = (__temp22 = g_strdup (name), (self->priv->dup_function == NULL ? NULL : (self->priv->dup_function = (g_free (self->priv->dup_function), NULL))), __temp22);
}


static char* vala_struct_real_get_free_function (ValaDataType* base)
{
	ValaStruct * self = VALA_STRUCT (base);
	if (self->priv->free_function == NULL) {
		char* __temp24 = NULL;
		char* __temp23 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (self)), (__temp24 = g_strdup_printf ("The type `%s` doesn't contain a free function", (__temp23 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (self)))))));
		(__temp24 == NULL ? NULL : (__temp24 = (g_free (__temp24), NULL)));
		(__temp23 == NULL ? NULL : (__temp23 = (g_free (__temp23), NULL)));
	}
	return self->priv->free_function;
}


static void vala_struct_set_free_function (ValaStruct* self, char* name)
{
	g_return_if_fail (VALA_IS_STRUCT (self));
	g_return_if_fail (name != NULL);
	char* __temp26 = NULL;
	self->priv->free_function = (__temp26 = g_strdup (name), (self->priv->free_function == NULL ? NULL : (self->priv->free_function = (g_free (self->priv->free_function), NULL))), __temp26);
}


static char* vala_struct_real_get_type_id (ValaDataType* base)
{
	ValaStruct * self = VALA_STRUCT (base);
	if (self->priv->type_id == NULL) {
		char* __temp28 = NULL;
		char* __temp27 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (self)), (__temp28 = g_strdup_printf ("The type `%s` doesn't declare a type id", (__temp27 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (self)))))));
		(__temp28 == NULL ? NULL : (__temp28 = (g_free (__temp28), NULL)));
		(__temp27 == NULL ? NULL : (__temp27 = (g_free (__temp27), NULL)));
	}
	return self->priv->type_id;
}


static void vala_struct_set_type_id (ValaStruct* self, char* name)
{
	g_return_if_fail (VALA_IS_STRUCT (self));
	g_return_if_fail (name != NULL);
	char* __temp30 = NULL;
	self->priv->type_id = (__temp30 = g_strdup (name), (self->priv->type_id == NULL ? NULL : (self->priv->type_id = (g_free (self->priv->type_id), NULL))), __temp30);
}


static char* vala_struct_real_get_marshaller_type_name (ValaDataType* base)
{
	ValaStruct * self = VALA_STRUCT (base);
	if (self->priv->marshaller_type_name == NULL) {
		char* __temp32 = NULL;
		char* __temp31 = NULL;
		vala_report_error (vala_code_node_get_source_reference (VALA_CODE_NODE (self)), (__temp32 = g_strdup_printf ("The type `%s` doesn't declare a marshaller type name", (__temp31 = vala_symbol_get_full_name (vala_code_node_get_symbol (VALA_CODE_NODE (self)))))));
		(__temp32 == NULL ? NULL : (__temp32 = (g_free (__temp32), NULL)));
		(__temp31 == NULL ? NULL : (__temp31 = (g_free (__temp31), NULL)));
	}
	return self->priv->marshaller_type_name;
}


static void vala_struct_set_marshaller_type_name (ValaStruct* self, char* name)
{
	g_return_if_fail (VALA_IS_STRUCT (self));
	g_return_if_fail (name != NULL);
	char* __temp34 = NULL;
	self->priv->marshaller_type_name = (__temp34 = g_strdup (name), (self->priv->marshaller_type_name == NULL ? NULL : (self->priv->marshaller_type_name = (g_free (self->priv->marshaller_type_name), NULL))), __temp34);
}


ValaMethod* vala_struct_get_default_construction_method (ValaStruct* self)
{
	g_return_val_if_fail (VALA_IS_STRUCT (self), NULL);
	return self->priv->_default_construction_method;
}


void vala_struct_set_default_construction_method (ValaStruct* self, ValaMethod* value)
{
	g_return_if_fail (VALA_IS_STRUCT (self));
	ValaMethod* __temp37 = NULL;
	ValaMethod* __temp36 = NULL;
	self->priv->_default_construction_method = (__temp37 = (__temp36 = value, (__temp36 == NULL ? __temp36 : g_object_ref (__temp36))), (self->priv->_default_construction_method == NULL ? NULL : (self->priv->_default_construction_method = (g_object_unref (self->priv->_default_construction_method), NULL))), __temp37);
}


void vala_struct_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec)
{
	ValaStruct * self = VALA_STRUCT (object);
	switch (property_id) {
		case VALA_STRUCT_DEFAULT_CONSTRUCTION_METHOD:
		g_value_set_object (value, vala_struct_get_default_construction_method (self));
		break;
	}
}


void vala_struct_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec)
{
	ValaStruct * self = VALA_STRUCT (object);
	switch (property_id) {
		case VALA_STRUCT_DEFAULT_CONSTRUCTION_METHOD:
		vala_struct_set_default_construction_method (self, g_value_get_object (value));
		break;
	}
}


static void vala_struct_class_init (ValaStructClass * klass)
{
	g_type_class_add_private (klass, sizeof (ValaStructPrivate));
	G_OBJECT_CLASS (klass)->get_property = vala_struct_get_property;
	G_OBJECT_CLASS (klass)->set_property = vala_struct_set_property;
	G_OBJECT_CLASS (klass)->dispose = vala_struct_dispose;
	VALA_CODE_NODE_CLASS (klass)->accept = vala_struct_real_accept;
	VALA_DATA_TYPE_CLASS (klass)->get_cname = vala_struct_real_get_cname;
	VALA_DATA_TYPE_CLASS (klass)->get_lower_case_cname = vala_struct_real_get_lower_case_cname;
	VALA_DATA_TYPE_CLASS (klass)->get_upper_case_cname = vala_struct_real_get_upper_case_cname;
	VALA_DATA_TYPE_CLASS (klass)->is_reference_type = vala_struct_real_is_reference_type;
	VALA_DATA_TYPE_CLASS (klass)->is_reference_counting = vala_struct_real_is_reference_counting;
	VALA_DATA_TYPE_CLASS (klass)->get_dup_function = vala_struct_real_get_dup_function;
	VALA_DATA_TYPE_CLASS (klass)->get_free_function = vala_struct_real_get_free_function;
	VALA_DATA_TYPE_CLASS (klass)->get_type_id = vala_struct_real_get_type_id;
	VALA_DATA_TYPE_CLASS (klass)->get_marshaller_type_name = vala_struct_real_get_marshaller_type_name;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VALA_STRUCT_DEFAULT_CONSTRUCTION_METHOD, g_param_spec_object ("default-construction-method", "foo", "bar", VALA_TYPE_METHOD, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE));
}


static void vala_struct_init (ValaStruct * self)
{
	self->priv = VALA_STRUCT_GET_PRIVATE (self);
}


static void vala_struct_dispose (GObject * obj)
{
	ValaStruct * self = VALA_STRUCT (obj);
	(self->priv->type_parameters == NULL ? NULL : (self->priv->type_parameters = (g_list_foreach (self->priv->type_parameters, (GFunc) g_object_unref, NULL), g_list_free (self->priv->type_parameters), NULL)));
	(self->priv->constants == NULL ? NULL : (self->priv->constants = (g_list_foreach (self->priv->constants, (GFunc) g_object_unref, NULL), g_list_free (self->priv->constants), NULL)));
	(self->priv->fields == NULL ? NULL : (self->priv->fields = (g_list_foreach (self->priv->fields, (GFunc) g_object_unref, NULL), g_list_free (self->priv->fields), NULL)));
	(self->priv->methods == NULL ? NULL : (self->priv->methods = (g_list_foreach (self->priv->methods, (GFunc) g_object_unref, NULL), g_list_free (self->priv->methods), NULL)));
	(self->priv->cname == NULL ? NULL : (self->priv->cname = (g_free (self->priv->cname), NULL)));
	(self->priv->dup_function == NULL ? NULL : (self->priv->dup_function = (g_free (self->priv->dup_function), NULL)));
	(self->priv->free_function == NULL ? NULL : (self->priv->free_function = (g_free (self->priv->free_function), NULL)));
	(self->priv->type_id == NULL ? NULL : (self->priv->type_id = (g_free (self->priv->type_id), NULL)));
	(self->priv->lower_case_csuffix == NULL ? NULL : (self->priv->lower_case_csuffix = (g_free (self->priv->lower_case_csuffix), NULL)));
	(self->priv->marshaller_type_name == NULL ? NULL : (self->priv->marshaller_type_name = (g_free (self->priv->marshaller_type_name), NULL)));
	(self->priv->_default_construction_method == NULL ? NULL : (self->priv->_default_construction_method = (g_object_unref (self->priv->_default_construction_method), NULL)));
	ValaStructClass * klass;
	GObjectClass * parent_class;
	klass = VALA_STRUCT_CLASS (g_type_class_peek (VALA_TYPE_STRUCT));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	parent_class->dispose (obj);
}


GType vala_struct_get_type ()
{
	static GType g_define_type_id = 0;
	if (G_UNLIKELY (g_define_type_id == 0)) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaStructClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_struct_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaStruct), 0, (GInstanceInitFunc) vala_struct_init };
		g_define_type_id = g_type_register_static (VALA_TYPE_DATA_TYPE, "ValaStruct", &g_define_type_info, 0);
	}
	return g_define_type_id;
}



